package activity;

import java.io.IOException;
import interfaces.IActivityListener;
import interfaces.IActivityMonitor;
import interfaces.StatAggregator;
import protocol.Message;
import protocol.MsgType;
import protocol.TCPClientMessage;

public class ActivityListenerProxy extends TCPClientMessage implements Runnable,IActivityMonitor {

		
	private static final boolean DEBUG = true;
	private IActivityListener am;
	private Thread t;

	public ActivityListenerProxy(IActivityListener proxy) {
		am = proxy;		
	}
	
	public void connect() throws IOException {
		super.connect();
		t = new Thread(this);
		t.start();
	}
	
	public void run() {
		try {
			while (true) {
				waitUntilActive();
				notifyActivity();
				waitUntilInactive();
				notifyInactivity();
			}
		} catch (InterruptedException e) {
			if (DEBUG) 
				System.err.println("ActivityListenerProxy thread quitting");
		}
		t=null;
	}

	public void waitUntilActive() throws InterruptedException {
		try {
			sendMessage(new Message(MsgType.WAIT_UNTIL_ACTIVE));
			Message m = getMessage();
			assert m.head == MsgType.ACK_OK ;
		} catch (IOException e) {
			throw new InterruptedException("IO error "+e);
		}				
	}

	public void waitUntilInactive() throws InterruptedException {
		try {
			sendMessage(new Message(MsgType.WAIT_UNTIL_INACTIVE));
			Message m = getMessage();
			assert m.head == MsgType.ACK_OK ;
		} catch (IOException e) {
			throw new InterruptedException("IO error "+e);
		}				
	}

	public boolean isActive() {
		try {
			sendMessage(new Message(MsgType.WAIT_UNTIL_INACTIVE));
			Message m = getMessage();
			if (m.head == MsgType.ACK_OK )
				return true;
			else if (m.head == MsgType.ACK_OK )
				return false;
			else {
				assert false;
				return false;
			}
		} catch (IOException e) {
			System.err.println("Proxy failed due to disconnected error");
			e.printStackTrace();
			return false;
		}	
	}

	public void notifyActivity() {
		am.notifyActivity();
	}

	public void notifyInactivity() {
		am.notifyInactivity();
	}

	public void getStats(StatAggregator stats) {
		StringBuffer ret = new StringBuffer("\n ActivityListenerProxy :" + super.toString());
		if (t!=null) ret.append(" relay thread:"+t.getState());
		else ret.append("no active relay thread");
		stats.agregate(ret);
	}

	public void close() {
		t.interrupt();
		disconnect();
	}
	
//	public boolean isEmpty() {
//		try {
//			Vector<MsgType> v = new Vector<MsgType> (1);
//			v.add(MsgType.ACK_OK);
//			v.add(MsgType.ACK_NOK);
//			
//			sendMessage(new Message(MsgType.IS_EMPTY_REQ));						
//			Message m = getMessageByType(v);
//			if (m.head == MsgType.ACK_OK)
//				return true;
//			else
//				return false;
//		} catch (Exception e) {
//			e.printStackTrace();
//			throw new NullPointerException ("Disconnected !");
//		} 
//	}
//
//	public IState popState() throws InterruptedException {
//		throw new NullPointerException("Unimplemented method !"+Thread.currentThread().getStackTrace());
//	}
//
//	public void pushState(IState s) {
//		throw new NullPointerException("Unimplemented method !"+Thread.currentThread().getStackTrace());
//	}

//	
//	public void getStats(StatAggregator stats) {
//		try {
//			Vector<MsgType> v = new Vector<MsgType> (1);
//			v.add(MsgType.STATS_RESP);
//			
//			sendMessage(new Message(MsgType.STATS_REQ));
//			Message m = getMessageByType(v);
//			assert(m.head == MsgType.STATS_RESP);
//			stats.agregate(m.data.get(0));
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (InterruptedException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//	}

	

}
