package basic;

import interfaces.IFiringManager;
import interfaces.IModelEngine;
import interfaces.IModelFactory;
import interfaces.IState;
import interfaces.IStateManager;
import interfaces.IStateSet;
import interfaces.StatAggregator;

import java.util.Enumeration;

public class FiringManagerThread extends Thread implements IFiringManager {
	
	
	private IStateSet nss_;
	private IStateManager sm_;
	private IModelEngine me_;
	private int nbprocessed;

	public FiringManagerThread(IStateSet nss, IStateManager sm, IModelFactory mf) {
		nss_ = nss;
		sm_ = sm;
		me_ = mf.create();
		nbprocessed = 0;
	}

	public void startGeneration() {
		nss_.notifyActivity();
		start();
	}

	public void run() {
		while (true) {
			try {
				IState s = nss_.popState();
				Enumeration<IState> it = me_.getSuccessors(s);
				while ( it.hasMoreElements() ) {
					sm_.processState(it.nextElement());
					++nbprocessed;
				}								
			} catch (InterruptedException e) {
				return ;
			}
		}
	}
	
	public void processInitialState() {
		sm_.processState(me_.getInitial());
	}

	public void getStats(StatAggregator stats) {
		stats.agregate("\nFiring manager Thread "
				+ " Processed=" + nbprocessed ) ;
	}

	public void close() {
		interrupt();
		try {
			this.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}
