package basic;

import interfaces.IActivityMonitor;
import interfaces.IStatistics;
import interfaces.StatAggregator;

public class IntegerActivityMonitor implements IActivityMonitor,IStatistics {

	int users_;
	Boolean mutex;
	private static final boolean DEBUG = true;

	public IntegerActivityMonitor () {
		users_ = 0;
		mutex = new Boolean(true);
	}

	public void notifyActivity() {
		synchronized (mutex) {
			if (++users_ == 1)
				mutex.notifyAll();
			if (DEBUG) System.err.println(this);
		}
	}


	public void notifyInactivity() {
		synchronized (mutex) {
			if (--users_ == 0)
				mutex.notifyAll();
			if (DEBUG) System.err.println(this);
		}
	}


	public void waitUntilActive() throws InterruptedException {
		synchronized (mutex) {
			while (users_ == 0) 
				mutex.wait();
		}
	}


	public void waitUntilInactive() throws InterruptedException {
		synchronized (mutex) {
			while (users_ > 0) 
				mutex.wait();
		}
	}


	public boolean isActive() {
		synchronized (mutex) {
			return users_ > 0;
		}
	}

	public String toString () {
		return "IntegerActivityMonitor : active=" + users_ ;
	}

	public void getStats(StatAggregator stats) {
		stats.agregate("\n  "+this); 		
	}

	public void close()
	{
		if (mutex != null) {
			synchronized(mutex) {
				mutex.notifyAll();
				mutex = null;
				users_ = 0;
			}
		}
	}

}
