package cami;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import net.TCPClient;
import protocol.IMessageConnection;

/** Handle basic low-level packet exchange withFrameKit. */
public class FKSocket extends TCPClient implements IMessageConnection<FKCommand> {

	/** output stream */
	private DataOutputStream out;

	/** input stream */
	private DataInputStream in;

	/** input buffer */
	byte [] buffer;
	int bufpos = 0;

	public synchronized void connect () throws IOException {
		super.connect();
		out = new DataOutputStream(sock.getOutputStream());
		in = new DataInputStream(new BufferedInputStream(sock.getInputStream()));	
	}

	public FKCommand getMessage() throws IOException {
		if (buffer == null || bufpos >= buffer.length-1) {
			// fill buffer
			// java is bigendian, like framekit
			int len = in.readInt();
			buffer = new byte[len];
			// blocking !
			for (int off = 0; off < len ; ) {
				off += in.read(buffer,off,len- off);
			}
			bufpos = 0;
		}
		FKCommand fkcmd;
		try {
			String msg = "";
			for (; bufpos < buffer.length; bufpos++) {
				byte b = buffer[bufpos];
				if (b == '\n') {
					bufpos++;
					break;
				}
					
				msg += (char)b;
			}
			fkcmd = new FKCommand(msg);
			if (fkcmd.code == CAMI.FC || fkcmd.code == CAMI.KO || fkcmd.code == CAMI.EOS)
				disconnect();
			System.out.print("<");
			System.out.println("<--:"+fkcmd);
			return fkcmd;
		}  catch (NullPointerException e) {
			throw new IOException("Protocol Message layer error :"+e);
		}	
	}


	public void sendMessage(FKCommand fkcmd) throws IOException {
		out.write(fkcmd.toPacket());
		System.out.print(">");
		System.out.println("-->:"+fkcmd);		
	}
	
	public void sendRawMessage (String s) throws IOException {
		out.write(FKCommand.toPacket(s));
	}


	public void disconnect() {
		try {
			sendMessage(new FKCommand(CAMI.FS,1));
			out.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
		super.disconnect();
	}

}
