package cluster;

import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import simple.SimpleModelFactory;

import activity.ActivityListenerMultiProxy;
import basic.StateSetArrayAdapter;
import gui.Configuration;
import interfaces.AbstractController;
import interfaces.IActivityMonitor;
import interfaces.IController;
import interfaces.IFiringManager;
import interfaces.IModelFactory;
import interfaces.IStateManager;
import interfaces.IStateSet;

/**
 * @author yann
 *
 */
public class MasterController extends AbstractController implements
		IController {

	
	protected MasterController(IModelFactory mf) {
		super(mf);
		initialize();
	}

	protected IActivityMonitor createActivityMonitor() {
		return new ActivityListenerMultiProxy (Configuration.getConf().getNbNodes());
	}
	
	protected IFiringManager createFiringManager() {
		return new FiringManagerMultiProxy(Configuration.getConf().getNbNodes());
	}

	
	protected IStateManager createStateManager() {
		return new StateManagerMultiProxy(Configuration.getConf().getNbNodes());
	}

	
	protected IStateSet createStateSet() {
		return new StateSetArrayAdapter();
	}
	
	public void close() {
		super.close();
		return;
	}

	public static void main(String[] args) {
		Configuration c;

		try {
			c = new Configuration();
			c.loadConfiguration("default.dmc");
			Configuration.setConf(c);
		} catch (IOException e) {
			e.printStackTrace();
		}

		final  IController control = new MasterController(new SimpleModelFactory());

//		Timer t = new Timer(true);
//		class StatThread extends TimerTask {
//			@Override
//			public void run() {				
//				System.out.println(control.getStatistics());			
//			}
//
//		}
//
//		TimerTask tt = new StatThread();
//		t.schedule(tt,3000,3000);

		Calendar beg = new GregorianCalendar();
		control.generateStateSet();
		Calendar end = new GregorianCalendar();
		System.out.println("Elapsed :"+ (end.getTimeInMillis() - beg.getTimeInMillis()));
		
		System.out.println("Finished ! \n Stats : " + control.getStatistics());
		
		control.close();
		//t.cancel();
	}
}
