package cluster;

import java.io.IOException;

import multi.FiringManagerMultiThread;

import simple.SimpleModelFactory;

import basic.StateSetArrayAdapter;

import dhash.DistributedHashTable;
import dhash.StateManagerService;

import activity.ActivityListenerService;
import gui.Configuration;
import interfaces.AbstractController;
import interfaces.IController;
import interfaces.IFiringManager;
import interfaces.IModelFactory;
import interfaces.IStateManager;
import interfaces.IStateSet;

public class PeerController extends AbstractController {

	private ActivityListenerService als_;
	private StateManagerService sms_;
	private FiringManagerService fms_;


	public PeerController(IModelFactory mf) throws IOException {
		super(mf);
		initialize();
		// nss and activity service
		als_ = new ActivityListenerService (nss_);
		als_.startServer(Configuration.getConf().getAMPort());
		// sm service		
		sms_ = new StateManagerService (sm_);
		try {
			sms_.startServer(Configuration.getConf().getSMPort());
		} catch (IOException e) {
			System.err.println("Could not start sm server on port"+ Configuration.getConf().getSMPort());
			e.printStackTrace();
		}
		fms_ = new FiringManagerService (fm_);
		fms_.startServer(Configuration.getConf().getFMPort());
	}

	@Override
	protected IFiringManager createFiringManager() {
		return new FiringManagerMultiThread (nss_,sm_,mf_,4);
		//return new FiringManagerThread(nss_,sm_,mf_);
	}

	@Override
	protected IStateManager createStateManager() {
		return new DistributedHashTable (Configuration.getConf().getNbNodes(),
										nss_);
	}

	@Override
	protected IStateSet createStateSet() {
		return new StateSetArrayAdapter();
	}

	public void close() {
		super.close();
		try {
			als_.stopServer();		
			sms_.stopServer();
			fms_.stopServer();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	public static void main(String[] args) {
		Configuration c;

		try {
			c = new Configuration();
			c.loadConfiguration("default.dmc");
			c.setLocalNode(Integer.parseInt(args[0]));
			Configuration.setConf(c);
		} catch (IOException e) {
			e.printStackTrace();
		}

		IController control;
		try {
			control = new PeerController(new SimpleModelFactory());
//			Timer t = new Timer(true);
//			class StatThread extends TimerTask {

//			@Override
//			public void run() {
//			System.out.println(control.getStatistics());			
//			}

//			}

//			TimerTask tt = new StatThread();
//			t.schedule(tt,3000,3000);
			System.out.println("slave node, press any key to exit");
			try {
				System.in.read();
			} catch (IOException e) {				
				e.printStackTrace();
			} 
			System.out.println("Locally finished ! \n Stats : " + control.getStatistics());
			control.close();
			control = null;
		} catch (IOException e1) {
			e1.printStackTrace();
		}	
		
	}



}
