package gui;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;

public class Configuration  {
	
	// singleton pattern
	static Configuration inst_ = null; 
	
	Vector<NodeConfiguration> hostConfs ;
	private int localId ;
	
	public Configuration () {
		hostConfs = new Vector<NodeConfiguration>();
		localId = 0;
	}
	
	public static Configuration getConf () {
		if (inst_ == null) {
			inst_ = new Configuration ();
		}
		return inst_;
	}
	
	public String asString () {
		StringBuffer b = new StringBuffer();
		b.append("Configuration :  localId="+localId + " nbNodes="+ hostConfs.size() + "\n " );
		int i = 0;
		for (NodeConfiguration n : hostConfs) {
			b.append( i++ + " : "  + n + "\n");
		}
		return b.toString();
	}
	
	public void loadConfiguration(String path) throws IOException {
		ObjectInputStream is = new ObjectInputStream(new FileInputStream(path));
		try {
			localId = is.readInt();
			hostConfs = (Vector<NodeConfiguration>) is.readObject();
		} catch (ClassNotFoundException e) {
			throw new IOException("Bad file save format !");
		}
	}
	
	public void saveConfiguration (String path) throws IOException {
		ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(path));
		os.writeInt(localId);
		os.writeObject(hostConfs);
	}
	
	public InetAddress getHostIP(int index) {
		return hostConfs.get(index).ip;
	}
	
	public int addHostIP(String host) throws UnknownHostException {
		InetAddress a = InetAddress.getByName(host);
		if (!hostConfs.contains(a)) {
			if (a.equals(InetAddress.getLocalHost())) {
				localId = hostConfs.size();
			}
			NodeConfiguration n = new NodeConfiguration(a);
			hostConfs.add(n);
			return hostConfs.size()-1;
		}
		return hostConfs.indexOf(a);
	}

	public int getSMPort() {
		if (localId < 0) 
			return 0;
		return hostConfs.get(localId).smPort ;
	}

	public int getLocalId() {
		return localId;
	}

	public int getNbNodes() {
		return hostConfs.size();
	}

	public int getNSSPort() {
		if (localId < 0) 
			return 0;
		return hostConfs.get(localId).nssPort;
	}

	public int getFMPort() {
		if (localId < 0) 
			return 0;
		return hostConfs.get(localId).fmPort;
	}

	public int getAMPort() {
		if (localId < 0) 
			return 0;
		return hostConfs.get(localId).amPort;
	}

	public int getSMPort(int index) {
		return hostConfs.get(index).smPort;
	}

	public int getFMPort(int index) {
		return hostConfs.get(index).fmPort;
	}
	public int getAMPort(int index) {
		return hostConfs.get(index).amPort;
	}
	public int getNSSPort(int index) {
		return hostConfs.get(index).nssPort;
	}

	public NodeConfiguration getNodeConf(int index) {
		return hostConfs.get(index);
	}

	public static void setConf(Configuration c) {
		inst_ = c;
	}

	public void setLocalNode(int i) {
		localId = i;		
	}
	

}
