package ipn;

/** An IPN instance.
 * 
 * An instance is defined by its type and name.
 * An index (within a set of instances) is also provided for more efficiency in state updates.
 *  
 * @author yann
 *
 */
public class IPNInstance implements Comparable<IPNInstance>{
		
	private String name;
	private IPNType type;
	private int index;

	public IPNInstance(String name, IPNType type, int index) {
		this.name = name;
		this.type = type;
		this.index = index;
	}
	
	/** The name of this instance */
	public String getName() {
		return name;
	}
	
	/** The type of this instance */
	public IPNType getType() {
		return type;
	}
	
	/** The index of this instance */
	public int getIndex() {
		return index;
	}

	/** Comparable interface */
	public int compareTo(IPNInstance o) {		
		int c = name.compareTo(o.name);
		if (c == 0) {
			c = type.getName().compareTo(o.type.getName());
		}
		return c;
	}
	
	/** hashable interface */
	public int hashCode() {
		return name.hashCode() ^ type.hashCode();
	}
}
