package ipn;

import libpetri.nets.IPTNet;

/** A transition of an IPN.
 * 
 * IPN transitions are identified by their name and visibility.
 * A support for flattening the representation is also provided; it only needs to be implemented in unitary types.
 * 
 * @author yann
 *
 */
public interface IPNTransition {
	/** The name of this transition.*/
	public String getName();
	
	/** The visibility : public or private of this transition. */
	public Visibility getVisibility ();

	/** Support for flattening this transition.
	 * 
	 * Need only be supported by Unitary transitions.
	 * 
	 * @param receiver : The net in which the transition is flattened. 
	 * @param tname : The name of this transition in the receiver net. 
	 * @param path : The qualified path to this instance of the transition. 
	 * Is used to qualify the name of target places, as by definition they belong to the same instance.
	 */
	public void flatten(IPTNet receiver, String tname, String path);
}
