package ipn.composite;

import ipn.IPNInstance;
import ipn.IPNState;
import ipn.SynchroSet;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;
import java.util.Map.Entry;

public class BasicSynchronization implements Iterable<Map.Entry<IPNInstance, String>>{

	private Map<IPNInstance, Collection<String> > parts = new TreeMap<IPNInstance, Collection<String>>();


	public void addPart(IPNInstance i, String slaveName) {
		Collection<String> events = parts.get(i);
		if (events == null) {
			events = new Vector<String>();
			parts.put(i, events);
		}
		events.add(slaveName);
	}


	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (Map.Entry<IPNInstance, String> e: this) {
			sb.append("\t  "+e.getKey().getName()+"."+e.getValue()+" ;\n");
		}
		return sb.toString();
	}

	public void addParts(BasicSynchronization s) {
		for (Map.Entry<IPNInstance, String> e: s) {
			addPart(e.getKey(), e.getValue());
		}
	}

	
	public Collection<IPNState> getSuccessors(CompositeState source) {
		Vector<IPNState> succ = new Vector<IPNState> ();
		succ.add(source.copy());
		for (Map.Entry<IPNInstance,Collection<String>> e: parts.entrySet()) {
			IPNInstance inst = e.getKey();
			IPNState sourceSub = source.getSubstate(inst);
			Collection<IPNState> substates = inst.getType().getSuccessors(sourceSub, e.getValue());
			
			if (substates.isEmpty()) {
				// exists a locally non fireable part
				return new Vector<IPNState> ();
			} else {
				// for each successor in current set, add a variant in which inst has state chosen in substates
				Vector<IPNState> newsucc = new Vector<IPNState> ();
				for (IPNState ss : succ) {
					CompositeState s = (CompositeState) ss;
					for (IPNState sub : substates) {
						CompositeState copy = s.copy();
						copy.setSubstate(inst, sub);
						newsucc.add(copy);
					}
				}
				succ = newsucc;
			}
		}
		return succ;
	}

	/** for flattening. */
	public Collection<SynchroSet> getSynchroSet (String prefix) {
		Collection<SynchroSet> ret = new Vector<SynchroSet>();
		ret.add(new SynchroSet());
				
		for (Map.Entry<IPNInstance,Collection<String>> e: parts.entrySet()) {
			IPNInstance inst = e.getKey();
			Collection<SynchroSet> local = inst.getType().getSynchroSet(e.getValue(),prefix +"." + inst.getName());
			for (SynchroSet inlocal : local)
				for (SynchroSet inret : ret) {
					inret.addAll(inlocal);
				}
		
		}
		return ret ;
	}
	
	public Iterator<Entry<IPNInstance, String>> iterator() {
		return new SyncIterator();
	}
	
	
	/*************** Iterator interface trivia *********************/

	private class SyncIterator implements Iterator<Map.Entry<IPNInstance,String>> {
		Iterator<Map.Entry<IPNInstance, Collection<String>>> it;
		Iterator<String> strit ;

		private class PartEntry implements Map.Entry<IPNInstance,String> {
			IPNInstance key;
			String value;

			public IPNInstance getKey() {
				return key;
			}
			public String getValue() {
				return value;
			}
			public String setValue(String value) {				
				String old = this.value;
				this.value = value;
				return old;
			}
			public void setKey(IPNInstance key) {
				this.key = key;
			}
		}

		PartEntry current = new PartEntry();

		public SyncIterator() {
			it = parts.entrySet().iterator();
			strit = null;
		}

		public boolean hasNext() {
			if (strit != null && strit.hasNext()) 
				return true;
			else if (it.hasNext()) {
				Map.Entry<IPNInstance, Collection<String>> cur = it.next();
				current.setKey(cur.getKey());
				strit = cur.getValue().iterator();
				return hasNext();
			} else {
				return false;
			}
		}

		public Entry<IPNInstance, String> next() {					
			current.setValue(strit.next());
			return current;
		}

		public void remove() {
			throw new UnsupportedOperationException();
		}
	}


}

