package ipn.elementary;

import java.util.Vector;

import libpetri.nets.IPTNet;

import ipn.IPNState;
import ipn.IPNTransition;
import ipn.Visibility;

public class UnitaryTransition implements IPNTransition {

	private String name;
	private Visibility vis;
	private Vector<Arc> inhibitor = new Vector<Arc>();
	private Vector<Arc> input= new Vector<Arc>();
	private Vector<Arc> output= new Vector<Arc>();


	public UnitaryTransition(String name, Visibility vis) {
		this.name = name;
		this.vis = vis;
	}

	public String getName() {
		return name;
	}

	public Visibility getVisibility() {
		return vis;
	}

	public void addArc(Place p, ArcType at, int value) {
		Arc a = new Arc(p,at,value);
		switch (at) {
		case INPUT:
			input.add(a);
			break;
		case OUTPUT:
			output.add(a);
			break;
		case INHIBITOR:
			inhibitor.add(a);
			break;
		}
	}

	public IPNState fire(Marking m) {
		for (Arc a : input) {
			int old = m.getPlaceState(a.p);
			assert old - a.value >= 0;
			m.setPlaceState(a.p, old - a.value);
		}
		for (Arc a : output) {
			int old = m.getPlaceState(a.p);
			m.setPlaceState(a.p, old + a.value);
		}
		return m;
	}

	public boolean isInhibited(Marking m) {
		for (Arc a : inhibitor) {
			if (m.getPlaceState(a.p) >= a.value)
				return true;
		}
		return false;
	}

	public boolean isInputEnabled(Marking m) {
		for (Arc a : input) {
			int old = m.getPlaceState(a.p);
			int n = old - a.value;
			if (n < 0)
				return false;
			m.setPlaceState(a.p, n);
		}
		return true;
	}

	public String toString() {		
		StringBuffer sb = new StringBuffer();
		sb.append("\t"+vis.toString().toLowerCase()+" transition "+name + " {\n");
		for (Arc a:inhibitor) {
			sb.append(a);
		}
		for (Arc a:input) {
			sb.append(a);
		}
		for (Arc a:output) {
			sb.append(a);
		}
		sb.append("\t}\n");
		return sb.toString();
	}

	public void flatten(IPTNet receiver, String tname, String path) {
		for (Arc a:inhibitor) {
			receiver.createArc(path+"."+a.p.getName(), tname, libpetri.structure.arcs.ArcType.INHIBITOR);
		}		
		for (Arc a:input) {
			receiver.createArc(path+"."+a.p.getName(), tname, libpetri.structure.arcs.ArcType.INPUT);
		}		
		for (Arc a:output) {
			receiver.createArc(path+"."+a.p.getName(), tname, libpetri.structure.arcs.ArcType.OUTPUT);
		}
	}

}
