package libpetri.dot;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Vector;
import java.util.regex.Pattern;

import libpetri.graphics.AnnotationGraphics;
import libpetri.graphics.ArcGraphics;
import libpetri.graphics.NodeGraphics;
import libpetri.nets.ISymmetricNet;
import libpetri.structure.ITransition;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;

public class DOTnet implements ISymmetricNet {
	
	// to store the named element to node number for cross references (arcs)
	TreeMap <String,DOTNode> nodes;
	Vector <DOTArc> arcs;
	// macao nodes index start from 2. Node 1 is declaration node.
	int lastIndex = 1;
	PrintStream out;


	public DOTnet(String name, String path) throws FileNotFoundException {
		nodes = new TreeMap<String, DOTNode>();
		arcs= new Vector<DOTArc>();
		// replace any non alpha character by an underscore
		name = Pattern.compile("\\W").matcher(name).replaceAll("_");
		out = new PrintStream (new FileOutputStream(path+name+".dot"));
		out.println("digraph \""+name + "\" {");
		out.println("\tsize=\"10,10\";\n");
		out.println("\toverlap=\"scale\";\n");
	}
	
	public DOTnet(String name) throws FileNotFoundException {
		this(name,name+".dot");
	}

	public void close () {
		for (DOTNode d:nodes.values()) { 
			out.println(d.getDOTString());
		}
		for (DOTArc d:arcs) { 
			out.println(d.getDOTString());
		}
		out.println(" }");
	}
	
	public boolean createArc(String pname, String tname, ArcType at, int value) {
		return createArc(pname,tname,at, Integer.toString(value));
	}
	
	public boolean setPlaceState(String name, int mark) {
		return setPlaceState(name,Integer.toString(mark));
	}
	
	public boolean createArc(String pname, String tname, ArcType at, String func) {
		if (at == ArcType.INPUT)
			arcs.add(new DOTArc(pname,tname,func));
		else
			arcs.add(new DOTArc(tname,pname,func));
		return false;
	}

	public boolean createPlace(String name) {
		nodes.put(name, new PlaceNode(name));
		return false;
	}

	public boolean createTransition(String name, Visibility vis) {
		nodes.put(name, new TransNode(name,vis));
		return false;
	}

	public void flatten(ISymmetricNet receiver) {
		throw new UnsupportedOperationException();
	}

	public Collection<ITransition> getPublicTransitions() {
		throw new UnsupportedOperationException();
	}

	public boolean setPlaceState(String name, String mark) {
		PlaceNode pn = (PlaceNode) nodes.get(name);
		pn.setMark(mark);
		return true;
	}

	public boolean createDomain(String domName, String domElts) {
		throw new UnsupportedOperationException();
	}

	
	public boolean createArc (String pname, String tname, ArcType at) {
		return createArc(pname,tname,at,1);
	}

	public boolean createPlace(String place, String domain) {
		throw new UnsupportedOperationException();
	}

	public boolean createTypeDef(String typename, String clinames) {
		throw new UnsupportedOperationException();
	}

	public boolean createArc(String pname, String tname, ArcType at, String func, ArcGraphics g) {
		throw new UnsupportedOperationException();
	}

	public boolean createArc(String pname, String tname, ArcType at, int value, ArcGraphics g) {
		throw new UnsupportedOperationException();
	}

	public AnnotationGraphics getNodeAnnotationGraphics(String nodeName, String attribName, AnnotationGraphics g) {
		throw new UnsupportedOperationException();
	}

	public NodeGraphics getNodeGraphics(String nodeName) {
		throw new UnsupportedOperationException();
	}

	public void setNodeAnnotationGraphics(String nodeName, String attribName, AnnotationGraphics g) {
		throw new UnsupportedOperationException();
	}

	public void setNodeGraphics(String nodeName, NodeGraphics g) {
		throw new UnsupportedOperationException();
	}
}

abstract class DOTNode {
	String id;
	String shape;
	public String getDOTString() {
		return "\t\""+id+ "\" [" +getLabel()+","+shape+"];";
	}
	abstract String getLabel();
}
class PlaceNode extends DOTNode {
	String mark;
	public PlaceNode(String id) {
		this.id = id;
		shape = "shape=ellipse";
	}
	public void setMark(String mark) {
		this.mark = mark;
	}
	String getLabel() {
		String[] name = id.split("_");
		String ret = "label=\""+name[name.length-1];
		if (mark != null && mark != "")
			ret += ",\\n"+mark;
		ret += "\"";
		return ret ;
	}
}
class TransNode extends DOTNode {
	public TransNode(String id, Visibility v) {
		this.id = id;
		if (v == Visibility.PUBLIC)
			shape = "shape=box,style=filled,color=\".7 .3 1.0\"";
		else
			shape = "shape=box";
	}
	String getLabel() {
		String[] name = id.split("_");
		return "label=\""+name[name.length-1]+"\"";
	}
}

class DOTArc {
	String src;
	String dest;
	String label;
	
	public DOTArc(String src, String dest, String func) {
		this.src= src;
		this.dest = dest;
		this.label = func;
	}

	public String getDOTString() {
		return "\t\""+src+"\" -> \""+dest+"\" [label=\""+label+"\"];";
	}
}
