package libpetri.namespaces;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import org.junit.Test;

public class TestNamespace {

	@Test
	public void testNamespace() {
		Namespace n = new Namespace("test");
		assertEquals(n.getFullyQualifiedName(), "test");
		assertEquals("/", Namespace.Root.getFullyQualifiedName() );		
	}

	@Test
	public void testAdd() {
		NamedElement ne = new NamedElement("toto");
		try {
			Namespace.Root.add(ne);
			assertEquals(Namespace.Root, ne.getNamespace());
			assertEquals("/.toto",ne.getFullyQualifiedName());					
		} catch (NamingException e) {
			e.printStackTrace();
			fail("Unexpected naming exception caught !");
		}
		try {
			Namespace.Root.add(ne);
		} catch (NamingException e) {
			fail("Double add not handled correctly");
		}
		try {
			NamedElement ne2 = new NamedElement("toto");
			Namespace.Root.add(ne2);
			fail("Should raise conflicting names exception");
		} catch (Exception e) {
			// looks good, caught conflicting names error
		}

	}

	@Test
	public void testRemove() {
		Namespace ns = new Namespace("ns");
		NamedElement ne = new NamedElement("testRem");
		try {
			ns.add(ne);
			assertEquals(true, ns.remove(ne));
			ns.add(ne);
			NamedElement ne2 = new NamedElement("testRem2");
			assertEquals(false, ns.remove(ne2));
		} catch (NamingException e) {
			e.printStackTrace();
			fail("Namespace add failed !");
		}
	}

	@Test
	public void testNamedElement() {
		NamedElement ne = new NamedElement("toto");
		assertEquals("toto", ne.getFullyQualifiedName());
	}

	@Test
	public void testSetNamespace() {
		NamedElement ne = new NamedElement("toto");
		Namespace ns = new Namespace("ns");
		try {
			ne.setNamespace(ns);
			assertEquals("ns.toto", ne.getFullyQualifiedName());
		} catch (NamingException e) {
			fail("set namespace failure");
			e.printStackTrace();
		}
	}

	@Test
	public void testGetFullyQualifiedName() {
		NamedElement ne = new NamedElement("toto");
		Namespace ns1 = new Namespace("ns1");
		try   {
			ns1.add(ne);
			assertEquals("ns1.toto", ne.getFullyQualifiedName());
			ns1.setNamespace(Namespace.Root);
			assertEquals("/.ns1.toto", ne.getFullyQualifiedName());
			assertEquals("/.ns1", ns1.getFullyQualifiedName());
			assertEquals("/", Namespace.Root.getFullyQualifiedName());
		} catch (Exception e) {
			fail("add failure");
		}
	}

	@Test
	public void testSetName() {
		try {
		NamedElement ne = new NamedElement("toto");
		assertEquals("toto", ne.getFullyQualifiedName());
		ne.setName("titi");
		assertEquals("titi", ne.getFullyQualifiedName());
		Namespace ns = new Namespace("ns");
		ne.setNamespace(ns);
		assertEquals("ns.titi", ne.getFullyQualifiedName());
		} catch (Exception e) {
			fail("unexpected exception caught !");
		}
	}

}
