package libpetri.structure;

import java.util.Vector;

import libpetri.namespaces.NamedElement;

public class Domain extends NamedElement{

	private static final Domain nullDom = new NullDomain();
	Vector<TypeDef> v;
	int [] dims;

	
	public Domain(String name) {
		super(name);
		v = new Vector<TypeDef>();
	}

	public void add(TypeDef clients) {
		v.add(clients);
		// invalidate cache
		dims = null;
	}

	public int[] getDims() {
		// use a cache
		if (dims == null) {
			dims = new int [v.size()];
			int i = 0;
			for (TypeDef t : v){
				dims[i++]=t.size();
			}
		}
		return dims;
	}

	public int size() {
		return v.size();
	}

	public static Domain getNullDomain() {
		return nullDom;
	}

	public TypeDef get(int i) {
		return v.get(i);
	}

	public String toString() {
		StringBuffer sb = new StringBuffer("domain "+getName()+ " is ");
		for (TypeDef t : v) {
			sb.append(t.getName()+", ");
		}
		sb.deleteCharAt(sb.length()-2);
		sb.append(";");
		return sb.toString();
	}
	
}

final class NullDomain extends Domain {

	NullDomain() {
		super("null");
	}
	
	public String toString() {
		return "";
	}
	
}