package libpetri.structure;
import java.util.Hashtable;
import java.util.Iterator;

import libpetri.graphics.AnnotationGraphics;
import libpetri.graphics.NodeGraphics;
import libpetri.namespaces.NamedElement;
import libpetri.nets.Indexable;
import libpetri.structure.arcs.Arc;
import libpetri.structure.arcs.ArcSet;

/** A first basis for node objects : places and transitions. 
 * 
 * Carries a namedElement + an index and a set of arc references.
 * 
 * */
public class Node extends NamedElement implements INode,Indexable {

	protected Hashtable<String,AnnotationGraphics> tagPositions;
	protected NodeGraphics graphics;
	protected ArcSet arcs;
	private int index;
		
	/** A node is a named element. */
	public Node(String name) {
		super(name);
		arcs = new ArcSet();
		tagPositions = new Hashtable<String, AnnotationGraphics>();
	}
	
	public void addArc(Arc a) {
		arcs.add(a);
	}
	
	public Iterator<Arc> getArcs () {
		return  arcs.iterator();
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int index) {
		this.index = index;
	}

	public Net getContainer() {
		return (Net) getNamespace();
	}
		
	
	public NodeGraphics getGraphics() {
		
		return graphics;
	}
	
	public void setGraphics(NodeGraphics graphics) {
		this.graphics = graphics;
	}
	
	public AnnotationGraphics getTagPosition (String tagName) {
		return tagPositions.get(tagName);
	}
	public void setTagPosition (String tagName, AnnotationGraphics g) {
		tagPositions.put(tagName,g);
	}
}
