package libpetri.structure.arcs;


import libpetri.graphics.ArcGraphics;
import libpetri.nets.ISymmetricNet;
import libpetri.structure.Place;
import libpetri.structure.Transition;
import libpetri.structure.state.State;

public abstract class Arc {

	public static Arc createArc(Place place, Transition trans, ArcType at, Function f) {
		switch (at) {
		case INPUT :
			return new InputArc(place, trans, f);
		case OUTPUT :
			return new OutputArc(place,trans, f);
		}
		return null;
	}

	protected Place place;
	protected Transition trans;
	protected Function func;
	private ArcGraphics graphics;
	
	protected Arc(Place place,Transition trans, Function func) {
		this.place = place;
		this.trans = trans;
		this.func = func;
	}

	public abstract boolean isEnabled(State s,int[]bindings);

	public abstract void fire(State s2,int[]bindings) ;

	public void flatten(ISymmetricNet receiver,String alias) {
		if (alias == null)
			receiver.createArc(place.getFullyQualified_Name(), 
					trans.getFullyQualified_Name(), 
					getType(),func.toString(place.getDom(),trans));
		else
			receiver.createArc(place.getFullyQualified_Name(), 
					alias, 
					getType(),func.toString(place.getDom(),trans));
	}
	
	public void convert(ISymmetricNet a) {
		a.createArc(place.getName(), 
					trans.getName(), 
					getType(), 
					func.toString(place.getDom(),trans));
	}

	public abstract  ArcType getType();
	
	public ArcGraphics getGraphics() {
		return graphics;
	}
	public void setGraphics(ArcGraphics graphics) {
		this.graphics = graphics;
	}
}
