package libpetri.structure.arcs;

import java.util.Vector;

import libpetri.structure.Domain;
import libpetri.structure.Element;
import libpetri.structure.Transition;
import libpetri.structure.Variable;
import libpetri.structure.state.PlaceState;
import libpetri.tensor.Tensor;


/** A function of integer variables into a tensor space.
* A binding (int[]) gives the value of each variable in order.
* Implem uses an array of tokens (int[]) in which positive coordinates 
*  are interpreted as constants, and negative values as references to variables.
**/
public class Function {

	Vector<Vector<Integer>> tokens;
	int [] dims;
	
	/** constructor : just give dimensions of place domain.
	 * 
	 * @param dims : the dimensions to initalize the tensors with. 
	 * Should be homogeneous to place tensor dims.
	 */
	public Function(int[]dims) {
		tokens = new Vector<Vector<Integer>> ();
		this.dims = dims;
	}
	/** return a tensor representing the evaluation of the function for this variable binding.
	 * 
	 * @param bindings : gives variable values : bindings[i] = value of var i
	 * @return a tensor that can be compared or added to a place marking 
	 */
	public PlaceState eval (int [] bindings) {
		Tensor t = new Tensor(dims);
		int [] coord = new int[dims.length];
		// foreach token
		for (Vector<Integer> tok : tokens) {
			int i = 0;
			// for each element of token
			for (Integer val: tok) {
				if (val < 0) {
					// -val-1 because var 0 is referred to by -1 not -0
					coord[i] = bindings[-val -1]; 
				} else {
					// copy constant
					coord[i] = val;
				}
				i++;
			}
			t.add(coord);
		}
		return new PlaceState(t);
	}
	
	public void add (int [] token, int card) {
		assert(card==1);
		Vector<Integer> tok = new Vector<Integer> ();
		for (int i:token)
			tok.add(i);
		tok.trimToSize();
		tokens.add(tok);
	}
	
	public String toString (Domain dom, Transition t) {
		if (tokens.isEmpty())
			return "";
		StringBuffer sb = new StringBuffer(128);
		for (Vector<Integer> coord : tokens) {
			if (coord.isEmpty())
				sb.append(" 1 ");
			else
				sb.append(" 1'< ");
			for (int cl = 0 ; cl < coord.size() ; cl++ ) {
				Element e = dom.get(cl).getByIndex(coord.get(cl));
				if (e==null) {
					Variable v = t.getVarByIndex(coord.get(cl));
					sb.append(v.getName());
				} else {
					sb.append(e);
				}
				sb.append(",");
			}
			if (! coord.isEmpty())
				sb.setCharAt(sb.length()-1, '>');
			sb.append(" +");
		}
		// kill the last + character
		return sb.substring(0, sb.length()-1);
	}
}
