package libpetri.structure.state;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import libpetri.structure.Domain;
import libpetri.structure.Place;
import libpetri.structure.TypeDef;

public class ParseMarking {


	public static PlaceState parse (Place p, String mark) {
		PlaceState ps = new PlaceState(p.getDom().getDims());


		Pattern pat = Pattern.compile("^\\s*(\\d+)\\s*('\\s*<\\s*(.*?)\\s*>\\s*){0,1}$" );

		String[] toks = mark.split("\\+");
		for (String tok : toks) {
			Matcher matcher = pat.matcher(tok);
//			boolean found = false;
			while (matcher.find()) {
				int [] coord = new int [p.getDom().getDims().length];
				int card = Integer.parseInt(matcher.group(1));
//				System.out.println("I found the text \""+matcher.group()+"\" starting at " +
//						"index "+matcher.start()+" and ending at index "+ matcher.end());
				boolean doAdd = (card !=0);
				assert(card==0 || card==1);
				if (matcher.group(3) != null) {
					String[] toktab = matcher.group(3).split("\\s*,\\s*");
					int i=0;
					for (String elt : toktab) {
						if (elt.equals("")) break;
//						System.err.println("elt : \"" + elt + "\"");
						coord[i] = p.getDom().get(i).getIndex(elt);
						if (coord[i] <0) {
							System.err.println("Marking parse error for place "+p.getFullyQualifiedName()+", unknown element \""+elt+"\" in token \"<" + matcher.group(3) + ">\" supposed to be of domain "+p.getDom());
							doAdd = false;
							break;
						}
						i++;
					}
					if (!doAdd)
						break;
					if ( i < p.getDom().size())  {
						System.err.println("Marking parse error for place "+p.getFullyQualifiedName()+" not enough elements in token \"<" + tok + ">\" supposed to be of domain "+p.getDom());
						doAdd = false;
						break;
					}
				}
				if (doAdd)
					ps.add(coord, card);
//				for (int i = 1; i <= matcher.groupCount(); i++) 
//					System.out.println("group " +i + " :"+matcher.group(i));
//				found = true;
			}

//			if(!found){
//				System.out.println("No match found.");
//			}
		}

		return ps;
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		TypeDef t = new TypeDef("test","aa,bb,cc,dd");
		Domain d = new Domain("tesd");
		d.add(t);
		d.add(t);
		d.add(t);
		Place p = new Place("toto",d);
		PlaceState ps = ParseMarking.parse(p,"3 '< aa,  bb,cc>+ 1'<aa,bb,aa>"); 
		System.out.println(ps);		
		Domain dn= Domain.getNullDomain();
		Place pn = new Place("titi",dn);
		PlaceState psn = ParseMarking.parse(pn,"1"); 
		System.out.println(psn);			
	}

}
