package libpetri.tensor;

import java.util.Arrays;
import java.util.BitSet;

import libpetri.structure.Place;

public class TensorSet {

	int [][] dims;
	int [] offsets;
	BitSet vals;

	public TensorSet() {
		dims = new int [0][];
		offsets = new int[1];
		offsets[0] = 0;
		vals = new BitSet();
	}
	
	public TensorSet(int[][] dims, int[] offsets, BitSet vals) {
		this.dims = dims;
		this.offsets = offsets;
		this.vals = vals;
	}

	// defined here because subclassing in package structure, just to define this function is a waste.
	// introduces a nasty dependency from tensor to structure, but this is deliberate.
	public void addPlace (Place p) {
		assert(p.getIndex()==dims.length+1);
		int[][] dims2 = new int [dims.length+1][];
		System.arraycopy(dims, 0, dims2, 0, dims.length);
		dims = dims2;
		dims[p.getIndex()] = p.getDom().getDims();
		int[] offsets2 = new int [offsets.length+1];
		System.arraycopy(offsets, 0, offsets2, 0, offsets.length);
		offsets = offsets2;
		offsets[p.getIndex()+1]= offsets[p.getIndex()]+new Tensor(dims[p.getIndex()]).size();
	}
	
	public Tensor get(int index) {
		return new Tensor(dims[index],vals.get(offsets[index], offsets[index+1]));
	}
	
	public void add(int index, Tensor t) {
		assert(t.dims==dims[index]);
		for (int adr = t.vals.nextSetBit(0); adr >= 0 ; adr = t.vals.nextSetBit(adr+1)) {
			vals.set(adr+offsets[index]);
		}
	}
	public void remove(int index, Tensor t) {
		assert(t.dims==dims[index]);
		for (int adr = t.vals.nextSetBit(0); adr >= 0 ; adr = t.vals.nextSetBit(adr+1)) {
			vals.clear(adr+offsets[index]);
		}		
	}
	
	public void set(int index,Tensor t) {
		assert(t.dims==dims[index]);
		vals.set(offsets[index], offsets[index+1],false);
		for (int adr = t.vals.nextSetBit(0); adr >= 0 ; adr = t.vals.nextSetBit(adr+1)) {
			vals.set(adr+offsets[index]);
		}
	}

	public TensorSet copy() {
		return new TensorSet(dims, offsets, (BitSet) vals.clone());
	}

	public int size() {
		return dims.length;
	}

	public int compareTo(TensorSet other) {
		return Tensor.compareBitSet(vals, other.vals);
	}


	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + Arrays.hashCode(dims);
		result = PRIME * result + Arrays.hashCode(offsets);
		result = PRIME * result + ((vals == null) ? 0 : vals.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final TensorSet other = (TensorSet) obj;
		if (!Arrays.equals(dims, other.dims))
			return false;
		if (!Arrays.equals(offsets, other.offsets))
			return false;
		if (vals == null) {
			if (other.vals != null)
				return false;
		} else if (!vals.equals(other.vals))
			return false;
		return true;
	}




	
	
}
