package libpetri.test;

import java.util.Calendar;

import libpetri.structure.Net;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;

public class ColoredPrivateClient extends Net {

	public ColoredPrivateClient(String name, boolean isPrivate) {
		super(name);

		// create a color class (typedef) clientID
		StringBuffer clinames = new StringBuffer(32);
		for (int i=0; i < 4 ; i++)
			clinames.append("c"+i+",");
		clinames.deleteCharAt(clinames.length()-1);
		createTypeDef("clientID",clinames.toString());

		// creation of domain : clients
		createDomain("clients","clientID");

		createPlace("idle","clients");
		createPlace("wait","clients");
		createPlace("readAck","clients");

		setPlaceState("idle","1'<c1> + 1'<c2> ");

		String tname = "sendReq";
		if (isPrivate)
			createTransition(tname,Visibility.PRIVATE);
		else
			createTransition(tname,Visibility.PUBLIC);
		createTransitionParameter(tname,"X","clientID");
		createArc("idle", tname, ArcType.INPUT, "1'<X>");
		createArc("wait", tname, ArcType.OUTPUT, "1'<X>");

		tname = "getAck";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"X","clientID");
		createArc("wait", tname, ArcType.INPUT, "1'<X>");
		createArc("readAck", tname , ArcType.OUTPUT, "1'<X>");

		tname = "doOperation";
		if (isPrivate)
			createTransition(tname,Visibility.PRIVATE);
		else
			createTransition(tname,Visibility.PUBLIC);
		createTransitionParameter(tname,"X","clientID");
		createArc("readAck", tname, ArcType.INPUT, "1'<X>");
		createArc("idle", tname, ArcType.OUTPUT, "1'<X>");


	}

	public static void main(String[] args) {
		ColoredPrivateClient n = new ColoredPrivateClient ("colored",true);
		{
			System.out.println(n);
			Calendar c = Calendar.getInstance();
			n.generateStateSpace();
			Calendar c2 = Calendar.getInstance();
			System.out.println(" elapsed : "+ (c2.getTimeInMillis()-c.getTimeInMillis())+"ms");
		}
	}

}
