package micropilot;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Vector;

import cami.CAMI;
import cami.FKCommand;
import cami.FKSocket;

public class FKProxy extends FKSocket {


	private ConnectionSettings settings;

	public FKProxy(ConnectionSettings set) throws UnknownHostException {
		settings = set;
		setServer(set.getHost(), set.getPort());
	}

	@Override
	public synchronized void connect() throws IOException {
		super.connect();
		/* Premiere partie : Le login et le password */
		sendMessage(new FKCommand(CAMI.SC,settings.getUserName(),settings.getPassword()));

		FKCommand resp = getMessage();
		if (resp.getCode() != CAMI.SC) {
			throw new IOException("Protocol error, expecting SC, got "+resp);
		}

		/* Deuxieme partie les informations sur l'API */
		sendMessage(new FKCommand(
				CAMI.OC
				,settings.getAPIName()
				,settings.getVersion()
				,settings.getUserName()
				,0));
		resp = getMessage();
		if (resp.getCode() != CAMI.OC) {
			throw new IOException("Protocol error, expecting OC, got "+resp);
		}
	}
	
	public synchronized boolean openSession(String sessionName, long date, String sessionFormalism) 
	throws IOException {
		// Composition de la commande OS

		
		// block required to initialize session
		sendMessage(new FKCommand(
				CAMI.OS
				,sessionName
				,(int)date
				,sessionFormalism));
		sendMessage(new FKCommand(CAMI.DI));
		sendMessage(new FKCommand(CAMI.CI,sessionName,1));
		sendMessage(new FKCommand(CAMI.FI));

		for (FKCommand resp = getMessage(); resp.getCode() != CAMI.QQ ; resp = getMessage() ) {
			// read and discard response
			continue;
		}

		return true;
	}
	
	public synchronized Vector<ServiceResult> runService (Service s) throws IOException {
		// Si le modele a ete mis a jour depuis le dernier appel de service on doit envoyer un <MS>
		// sendMessage(new FKCommand(CAMI.MS));
		
		sendMessage(new FKCommand(CAMI.DT));
		// Si le nom du pere est vide... Alors le service est directement sous la racine
		if (!s.getName().equals(s.getRoot())) {
			sendMessage(new FKCommand(
					CAMI.PQ,
					s.getRoot(),
					s.getParent(),
					1));
		}
		sendMessage(new FKCommand(
				CAMI.PQ,
				s.getRoot(),
				s.getName(),
				1));
		sendMessage(new FKCommand(CAMI.FT));

		return handleServiceResponse(s);
	}
	
	
	private Vector<ServiceResult> handleServiceResponse(Service s) throws IOException {
		Vector<ServiceResult> res = new Vector<ServiceResult>();
		IServiceHandler sh = s.getServiceHandler();
		for (FKCommand resp = getMessage(); resp.getCode() != CAMI.FR ; resp = getMessage() ) {
			switch (resp.getCode()) {
			case DF :
				sendModel();
				break;
			default :
				sh.handleMessage(resp, res);
			}
		}

		return res;
	}

	private void sendModel() throws IOException {
		sendMessage(new FKCommand(CAMI.DB));
		for (String s : settings.getModel().translate()) {
			sendRawMessage(s);
		}
		sendMessage(new FKCommand(CAMI.FB));
	}

	public static void main(String[] args) {
		FKProxy fk;
		try {
			fk = new FKProxy(new ConnectionSettings());
			fk.connect();
			fk.openSession("toto", 15, "AMI-Net");
			fk.runService(Service.SYNTAX_CHECKER);
			fk.runService(Service.STRUCTURAL_BOUNDS);
			fk.disconnect();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
	}

	public void connectSession() throws IOException {
		connect();
		openSession(settings.getModelName(), settings.getModel().getDate(), "AMI-Net");
	}
}
