package multi;

import interfaces.IFiringManager;
import interfaces.StatAggregator;

import java.util.Vector;

public abstract class FiringManagerMulti implements IFiringManager {

	private Vector<IFiringManager> pool;
	private int nbthreads;

	private IFiringManager getFM(int i) {
		IFiringManager fm = pool.get(i);
		if (fm != null)
			return fm;
		else {
			pool.set(i,createFiringManager(i));
			return pool.get(i);
		}
	}

	protected abstract IFiringManager createFiringManager(int i) ;

	public FiringManagerMulti( int nbthreads) {
		this.nbthreads = nbthreads;
		pool = new Vector<IFiringManager> (nbthreads);
		pool.setSize(nbthreads);
	}

	public void startGeneration() {
		for (int i = 0 ; i < nbthreads ; i++) {
			getFM(i).startGeneration();
		}
	}

	public void processInitialState() {
		getFM(0).processInitialState();
	}

	public void getStats(StatAggregator stats) {
		stats.agregate("\nMulti FM : nbThreads="+nbthreads + " pool contains:");
		for (IFiringManager fm : pool) {
			if (fm != null) {
				fm.getStats(stats);
			} else {
				stats.agregate(" null fm entry");
			}			
		}
	}


	public void close() {
		for (IFiringManager fm : pool) {
			fm.close();			
		}
	}	


}
