package net ;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

/** Cette classe abstraite permet de creer un serveur TCP/IP
    generique dont il faudra implementer la methode gereClient
    pour le specialiser. */
    
abstract public class TCPServer implements Cloneable, Runnable {
    
    Thread thread = null ;
    private volatile Thread blinker;
    
    private ServerSocket serveur = null ;   // != null si thread serveur
    private Socket client = null ;          // != null si thread client
    static int compteurClient = 0 ;
    protected int numeroClient = 0 ;
    static final private boolean DEBUG = true; 

    
    private int port;
    /** Lance le serveur attach  un port donn.
        @param port le port d'attachement.
        @exception IOException si le serveur n'a pas pu tre cr. */
    
    public synchronized void startServer( int port ) throws IOException{
        if( thread == null ){
        	if (port == 0) 
        		serveur = new ServerSocket() ;
        	else
        		serveur = new ServerSocket( port ) ;
            thread = new Thread( this ) ;
            thread.start() ;
            this.port = port;
        }
    }
    
    /** Stoppe le serveur.
        @exception IOException si le serveur n'a pas pu tre cr. */
    
    public synchronized void stopServer() throws IOException{
        if( serveur != null ){
            Thread.currentThread().interrupt();
            blinker = null;
            serveur.close() ;
        }
    }
    
    public void run(){
        if( serveur != null ){  // serveur
            Thread thisThread = Thread.currentThread();
            blinker = thisThread ;
            while( blinker == thisThread ){
                try{
                	if (DEBUG) {
                		System.err.println("TCPServer accepting connections on port"+serveur.getLocalPort());
                	}
                    Socket socketClient = serveur.accept() ;
                    TCPServer nouvelleSocket = (TCPServer)clone() ;
                    nouvelleSocket.serveur = null ;     // client
                    nouvelleSocket.client = socketClient ;
                    TCPServer.compteurClient++ ;
                    nouvelleSocket.numeroClient = TCPServer.compteurClient ;
                    System.out.println( nouvelleSocket.numeroClient ) ;
                    nouvelleSocket.thread = new Thread( nouvelleSocket ) ;
                    nouvelleSocket.thread.start() ;
                } catch (SocketException e) {
                	if (serveur.isClosed()) {
                		if (DEBUG) {
                			System.err.println("Server wait thread ending on demand");
                		}
                	}
                }
                catch( Exception e ){
                    System.out.println( e ) ;
                }
            }
        } else {                // traitement d'un client
        	handleClient(client) ;
        	try {
				client.close();
			} catch (IOException e) {
				// Exception = elle est dj ferme : tout va bien
			}
			if (DEBUG)
				System.out.println("Fin d'un Thread de traitement de client couche TCP");
        }
    }
    

    public int getPort () {
    	return port;
    }
    


    
    /** Specialise le serveur pour un client donn. Implmentez cette mthode
        pour spcialiser le serveur.
        @param client la socket pour communiquer avec un client */    
    abstract public void handleClient( Socket client ) ;
}