package protocol;

import java.io.IOException;
import java.util.Collection;

public class SelectiveMessageConnection implements Runnable,ISelectiveMessageConnection {
	private Boolean msgSetMutex;
	private boolean msgIsSet=false;
	private Message lastMessage;
	private Thread handler;
	private IMessageConnection<Message> mc_;
	
	SelectiveMessageConnection (IMessageConnection<Message> mc) {
		mc_ = mc;
		msgSetMutex = new Boolean(true);
		msgIsSet = false;
		lastMessage = null;
		handler = null;		
	}
 	
	public Message getMessageByType (Collection<MsgType> t) throws InterruptedException {
		synchronized (msgSetMutex) {
			while (true) {
				while (! msgIsSet )
					msgSetMutex.wait();
				if (t.contains(lastMessage.head)) {
					msgIsSet = false;
					msgSetMutex.notifyAll();
					return lastMessage;
				}
			}
		}
	}
	
	// capture notify events
	public void waitForMessages() throws IOException, InterruptedException {
		Message m ;
		m = mc_.getMessage();
		synchronized (msgSetMutex) {
			while (msgIsSet == true)
				msgSetMutex.wait();
			lastMessage = m;
			msgIsSet = true;
			msgSetMutex.notifyAll();
		}
	}
	
	public void startHandler() {
		if (handler == null) {
			handler = new Thread(this);
			handler.setDaemon(true);
			handler.start();
		}
	}
	
	public void run() {		
		while (true) {
			try {
				waitForMessages();
			} catch (Exception e) {
				e.printStackTrace();
				return ;
			}
		}
	}

	public void sendMessage(Message m) throws IOException {
		mc_.sendMessage(m);
	}
}
