package transform;

import ipn.IPNModel;
import ipn.Visibility;
import ipn.composite.CompositeNet;
import ipn.composite.SyncType;

public class IPNTransformer {

	
	
	/** add an isolated type to the model */
	public static void createIsolated(IPNModel m, IPNBehavior beh) {
		CompositeNet net = new CompositeNet(getIsolatedName(beh.getTypeName()));

		String instName	= "inst"; 
		net.createInstance(instName, beh.getTypeName(), m);
		net.setSubnetState("active", instName, "active");
		
		exportCallEnd(net,instName);

		
		// handle links
		for (IPNBehaviorLink link : beh.getLinks()) {
			makePrivate(net,instName,link);
		}
		
		
		m.addType(net);
	}


	public static void createConnected(IPNModel m, IPNBehavior beh, BehaviorTransformationTrace btt, int depth) {
		CompositeNet net = new CompositeNet(getConnectedName(beh.getTypeName(),depth));
		
		assert depth > 0;
		
		String instName	= "inst"; 
		net.createInstance(instName, beh.getTypeName(), m);
		net.setSubnetState("active", instName, "active");
		
		exportCallEnd(net,instName);
		
		String tname;
		int i = 0;
		// handle links
		for (IPNBehaviorLink link : beh.getLinks()) {
			IPNBehavior called = btt.get(link.getBehavior());
			
			if (called != null) {
				String instCall = "called"+i++;
				net.createInstance(instCall, getConnectedName(called.getTypeName(),depth-1), m);
				
				tname = link.getCall();
				net.createSynchronization(tname, Visibility.PRIVATE, SyncType.AND);
				net.addSyncPart(tname, instName, tname);
				net.addSyncPart(tname, instCall, "call");
			
				tname = link.getEnd();
				net.createSynchronization(tname, Visibility.PRIVATE, SyncType.AND);
				net.addSyncPart(tname, instName, tname);
				net.addSyncPart(tname, instCall, "end");
			} else {
				makePrivate(net, instName, link);
			}
		}
		
		
		
		m.addType(net);
	}


	private static void makePrivate(CompositeNet net, String instName, IPNBehaviorLink link) {
		String tname;
		
		tname = link.getCall();
		net.createSynchronization(tname, Visibility.PRIVATE, SyncType.AND);
		net.addSyncPart(tname, instName, tname);
		
		tname = link.getEnd();
		net.createSynchronization(tname, Visibility.PRIVATE, SyncType.AND);
		net.addSyncPart(tname, instName, tname);

	}
	
	private static void exportCallEnd(CompositeNet net, String instName) {
		// call and end interface
		String tname = "call";
		net.createSynchronization(tname, Visibility.PUBLIC, SyncType.AND);
		net.addSyncPart(tname, instName, tname);
		
		tname = "end";
		net.createSynchronization(tname, Visibility.PUBLIC, SyncType.AND);
		net.addSyncPart(tname, instName, tname);
	}

	
	static String getConnectedName(String typeName, int depth) {
		if (depth == 0)
			return getIsolatedName(typeName);
		return "connected"+depth+"-"+typeName;
	}

	
	
	static String getIsolatedName(String typeName) {
		return "isolated"+typeName;
	}

}
