package transform;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.Map;
import java.util.Vector;

import libpetri.ami.AMInet;
import libpetri.structure.Net;
import micropilot.ConnectionSettings;
import micropilot.FKProxy;
import micropilot.Level;
import micropilot.Service;
import micropilot.ServiceResult;

import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;

public class ServiceOrchestrator {


	private Configuration conf;

	private ConnectionSettings fkConf ;

	private SimpleTransformer st;

	public ServiceOrchestrator() {
		conf = new Configuration();
		fkConf = new ConnectionSettings();
		st = new SimpleTransformer();
	}


	public Configuration getConf() {
		return conf;
	}

	public ConnectionSettings getFkConf() {
		return fkConf;
	}

	public Map<NamedElement, Net> doTransformation(Model model) {
		return st.handleModel(model, conf);
	}


	public Map<NamedElement, Net> doTransformation() {
		fkConf.setModelFile(conf.getTempCamiPath());
		return st.transform(conf);
	}
	
	public Map<NamedElement, Net> doTransformation(Element e) {
		fkConf.setModelFile(conf.getTempCamiPath());
		return st.handleModelElement(e, conf);
	}
	
	public Collection<IVerificationResult> checkBoundsUML (Element model) throws IOException {

		// convert activity diagrams to petri nets
		Map<NamedElement, Net> nets = doTransformation(model);

		System.err.println("Built "+nets.size()+ " net types");
		// checker preconfiguration
		Vector<Service> services = new Vector<Service>();
		services.add(Service.SYNTAX_CHECKER);
		services.add(Service.STRUCTURAL_BOUNDS);

		Collection<IVerificationResult> restmp = new Vector<IVerificationResult>();
		
		// wrap verification results
		for (Net n : nets.values()) {
			for (IVerificationResult vr : callServiceList(n, services)) {
				if (vr.getSeverityLevel() != Level.STATISTIC)
					restmp.add(vr);
			}
		}
		return restmp;
	}

	public Collection<IVerificationResult> callServiceList (Net n,Collection<Service> serv) throws IOException{
		Vector<IVerificationResult> res= new Vector<IVerificationResult>();
		System.err.println("Producing AMI format net for "+n.getName());
		AMInet an = new AMInet();
		n.convert(an);
		
		
		File tmpff;
		
			tmpff = File.createTempFile(n.getName(), ".petri");

//			String pathcami = conf.getWorkingDir() + "/" + n.getName()+".petri";
//			conf.setCamiPath(pathcami);
//			fkConf.setModelFile(conf.getTempCamiPath());
			fkConf.setModelFile(tmpff.getAbsolutePath());

			an.writeNet(tmpff.getAbsolutePath());
			System.err.println("AMI production done for "+n.getName());
			System.err.println("connecting to FrameKit");
			
			FKProxy fk = new FKProxy(fkConf);
			fk.connectSession();
			System.err.println("Connection to FrameKit established");
			
			
			Trace<Element, String> trUML = st.getTrace();
			Trace<String, Integer> trCami = an.getTrace();
			ITrace<Element, Integer> trace = new ComposeTrace<Element, String, Integer>(trUML, trCami);


			for (Service s : serv) {
				System.err.println("Running service "+s.getName());
				boolean first = true;
				for (ServiceResult sr : fk.runService(s)) {
					if (first) {
						System.err.println("Service "+s.getName()+ " finished. Reading results.");
						first = false;
					}					
					res.add(new UMLVerificationResult(sr,trace));
				}
				System.err.println("Finished running service "+s.getName());
			}
			fk.disconnect();
			System.err.println("Disconnection from FrameKit.");
			
		
		return res;
	}


	public void saveConfiguration(String path) throws IOException {
		ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(path));
		os.writeObject(conf);
		os.writeObject(fkConf);
	}

	public void loadConfiguration(String path) throws IOException {
		ObjectInputStream is = new ObjectInputStream(new FileInputStream(path));
		try {
			conf = (Configuration) is.readObject();
			fkConf = (ConnectionSettings) is.readObject();
		} catch (ClassNotFoundException e) {
			throw new IOException("Bad file save format !");
		}
	}
}
