package transform.activity;


import ipn.IPNModel;
import ipn.Visibility;
import ipn.elementary.ArcType;
import ipn.elementary.UnitaryType;

import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.AcceptEventAction;
import org.eclipse.uml2.uml.Action;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityFinalNode;
import org.eclipse.uml2.uml.CallAction;
import org.eclipse.uml2.uml.CallBehaviorAction;
import org.eclipse.uml2.uml.ControlFlow;
import org.eclipse.uml2.uml.ControlNode;
import org.eclipse.uml2.uml.DecisionNode;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ForkNode;
import org.eclipse.uml2.uml.InitialNode;
import org.eclipse.uml2.uml.JoinNode;
import org.eclipse.uml2.uml.MergeNode;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.ReceiveSignalEvent;
import org.eclipse.uml2.uml.SendSignalAction;

import transform.BehaviorTransformationTrace;
import transform.Configuration;
import transform.IPNBehavior;
import transform.IPNBehaviorLink;
import transform.Trace;

public class ActivitytoIPNTransformer implements ActivityTransformer {

	private Trace<Element, String> trace;
	private Trace<Element, String> eventTrace = new Trace<Element, String>();
	private Trace<Element, String> inNames = new Trace<Element, String>();
	private Trace<Element, String> outNames = new Trace<Element, String>();	
	private IPNModel model;
	private BehaviorTransformationTrace btt;

	public ActivitytoIPNTransformer(Trace<Element, String> trace, IPNModel model,  BehaviorTransformationTrace btt) {
		this.trace = trace;
		this.model = model;
		this.btt = btt;
	}

	private String addPlace (UnitaryType net, Element e, String namePrefix, int mark) {
		String n = namePrefix + e.hashCode();
		net.createPlace(n);
		if (mark != 0)
			net.setPlaceState("active",n, mark);
		trace.addTrace(e, n);
		return n;
	}

	public void handleActivity(Activity act, Configuration conf) {
		UnitaryType net = new UnitaryType(act.getQualifiedName());
		IPNBehavior result = new IPNBehavior(net.getName());
		btt.put(act,result);
		System.out.println("_______________"+net.getName());
		int i = 0;
		// one pass to create relevant places and load inNames and outNames
		for (Element e : act.allOwnedElements()) {
//			System.out.println(e);
			if (e instanceof InitialNode) {
				addPlace (net,e,"initial",1);
				// no arc to Initial activity node !
				inNames.addTrace(e, null);
				outNames.addTrace(e, trace.getTarget(e));

				String tname = "call";
				net.createTransition(tname,Visibility.PUBLIC);
				net.createArc(trace.getTarget(e),tname , ArcType.OUTPUT , 1);
				eventTrace.addTrace(e, tname);
			} else if (e instanceof ActivityFinalNode) {
				String pname = addPlace (net,e,"final",0);
				inNames.addTrace(e, pname);
				// no arc from final activity node !
				outNames.addTrace(e, null);				

				String tname = "end";
				net.createTransition(tname,Visibility.PUBLIC);
				net.createArc(trace.getTarget(e),tname , ArcType.INPUT , 1);
				eventTrace.addTrace(e, tname);				

			} else if (e instanceof CallBehaviorAction) {
				CallBehaviorAction action = (CallBehaviorAction) e;
				String pin = addPlace (net,e,"pre"+action.getName(),0);				
				String pwait = addPlace (net,e,"wait"+action.getName(),0);
				String pout = addPlace (net,e,"post"+action.getName(),0);				
				
				String tcall = "call" + action.getBehavior().getQualifiedName() + e.hashCode();
				net.createTransition(tcall,Visibility.PUBLIC);
				net.createArc(pin,tcall , ArcType.INPUT , 1);
				net.createArc(pwait,tcall , ArcType.OUTPUT , 1);
				
				String treturn = "end" + action.getBehavior().getQualifiedName() + e.hashCode();
				net.createTransition(treturn,Visibility.PUBLIC);
				net.createArc(pwait,treturn , ArcType.INPUT , 1);
				net.createArc(pout,treturn , ArcType.OUTPUT , 1);				
				
				IPNBehaviorLink link = new IPNBehaviorLink(action.getBehavior(),tcall,treturn);
				result.getLinks().addBehaviorLink(link);
					// traces
				inNames.addTrace(e, pin);				
				outNames.addTrace(e, pout);
			} else if (e instanceof SendSignalAction || e instanceof ReceiveSignalEvent || e instanceof AcceptEventAction) {
				NamedElement action = (NamedElement) e;
				String pname1 = addPlace (net,e,"in"+ action.getName(),0);
				String pname2 = addPlace (net,e,"out"+ action.getName(),0);
				inNames.addTrace(e, pname1);
				outNames.addTrace(e, pname2);

				String tname = e.getClass().getSimpleName()+ e.hashCode();
				net.createTransition(tname,Visibility.PUBLIC);
				net.createArc(pname1,tname , ArcType.INPUT , 1);
				net.createArc(pname2,tname , ArcType.OUTPUT , 1);
				eventTrace.addTrace(e, tname);
			} else if (e instanceof Action) {
				// Default handling of actions
				Action action = (Action) e;
				String pname = addPlace (net,e,action.getName(),0);
				inNames.addTrace(e, pname);
				outNames.addTrace(e, pname);
			} else if (e instanceof ControlFlow) {
				ControlFlow t = (ControlFlow) e;
				// a control flow linking two control nodes produces an extra intermediate place
				if (t.getSource() instanceof ControlNode && t.getTarget() instanceof ControlNode ) {
					String pname = addPlace (net,e,"intermediate",0);					
				}
			}
		}

		// one pass to add transitions and handle control nodes
		int trnum = 0;
		for (Element e : act.allOwnedElements()) {
			if (e instanceof ControlFlow) {
				ControlFlow t = (ControlFlow) e;
				String srcn = trace.getTarget(t.getSource());
				String destn = trace.getTarget(t.getTarget());
				if (srcn == null || destn == null) {
					assert  t.getSource() instanceof ControlNode ||  t.getTarget() instanceof ControlNode;
					// any control flow that touches a control node will be handled separately
					continue;
				}
				// basic control flow from activity to activity
				String tname = "transition_"+trnum++ ;
				Visibility v = Visibility.PRIVATE;
				if (t.getName() != null)
					tname += t.getName();
				tname += e.hashCode();
				net.createTransition(tname,v);
				net.createArc(outNames.getTarget(t.getSource()),tname , ArcType.INPUT , 1);
				net.createArc(inNames.getTarget(t.getTarget()),tname , ArcType.OUTPUT , 1);
				eventTrace.addTrace(e, tname);
			} else if (e instanceof ControlNode) {
				ControlNode control = (ControlNode) e;
				if (control instanceof ForkNode || control instanceof JoinNode) {
					String tname = control.getClass().getSimpleName() + control.getName() + control.hashCode();
					net.createTransition(tname,Visibility.PRIVATE);
					eventTrace.addTrace(e, tname);
					for (ActivityEdge a : control.getIncomings()) {
						String pname = getSourcePlaceName (a);
						net.createArc(pname,tname , ArcType.INPUT , 1);
					}
					for (ActivityEdge a : control.getOutgoings()) {
						String pname = getTargetPlaceName(a);
						net.createArc(pname,tname , ArcType.OUTPUT , 1);
					}	
				} else if (control instanceof MergeNode ) {
					String tname = "Merge" + control.getClass().getSimpleName() + control.getName() + control.hashCode();
					int index=0;
					// Merge node => only one outgoing edge
					EList<ActivityEdge> set = control.getOutgoings();
					if (set ==null || set.size() != 1)
						throw new NullPointerException("Merge node with no output control flow detected.");
					String targetPlace = getTargetPlaceName(set.get(0));

					for (ActivityEdge in : control.getIncomings()) {
						String pname = getSourcePlaceName(in);
						String thistname = tname+in.hashCode();
						net.createTransition(thistname, Visibility.PRIVATE);
						net.createArc(pname, thistname , ArcType.INPUT , 1);
						net.createArc(targetPlace, thistname , ArcType.OUTPUT , 1);
					}
				} else if (control instanceof DecisionNode ) {
					String tname = "Decision" + control.getClass().getSimpleName() + control.getName() + control.hashCode();
					int index=0;
					// Decision node => only one incoming edge
					EList<ActivityEdge> set = control.getIncomings();
					if (set ==null || set.size() != 1) {
					//	throw new NullPointerException("Decision node with no input control flow detected.");
						System.err.println("Decision node with no input control flow detected. This is an error.");
						continue;
					}
					
					String sourcePlace = getSourcePlaceName(set.get(0));

					for (ActivityEdge in : control.getOutgoings()) {
						String pname = getTargetPlaceName(in);
						String thistname = tname+in.hashCode();
						if (in.getGuard() != null 
								&& ! in.getGuard().stringValue().equals("true")) {
							thistname += " [" + in.getGuard().stringValue() + "]";
						}
						net.createTransition(thistname, Visibility.PRIVATE);
						net.createArc(sourcePlace, thistname , ArcType.INPUT , 1);
						net.createArc(pname, thistname , ArcType.OUTPUT , 1);
					}
				}
			} // end ControlNode case 

		} // end 2nd pass
		
		model.addType(net);
	}

	private String getSourcePlaceName(ActivityEdge a) {
		String pname = outNames.getTarget(a.getSource());
		if (pname == null) {
			// probably an arc from another control node.
			pname = trace.getTarget(a);
			assert pname != null;
		}
		return pname;
	}

	private String getTargetPlaceName(ActivityEdge a) {
		String pname = inNames.getTarget(a.getTarget());
		if (pname == null) {
			// probably an arc to another control node.
			pname = trace.getTarget(a);
			assert pname != null;
		}
		return pname;
	}

}

