package transform.activity;

import java.util.Map;

import libpetri.structure.Net;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;

import org.eclipse.uml2.uml.AcceptEventAction;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityFinalNode;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.CallAction;
import org.eclipse.uml2.uml.ControlFlow;
import org.eclipse.uml2.uml.DecisionNode;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ForkNode;
import org.eclipse.uml2.uml.InitialNode;
import org.eclipse.uml2.uml.JoinNode;
import org.eclipse.uml2.uml.MergeNode;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OpaqueAction;
import org.eclipse.uml2.uml.ReceiveSignalEvent;
import org.eclipse.uml2.uml.SendSignalAction;

import transform.Configuration;
import transform.Trace;

public class ActivitytoNetTransformer implements ActivityTransformer {

	private Trace<Element, String> trace;
	private Map<NamedElement, Net> products;

	public ActivitytoNetTransformer(Trace<Element, String> trace,
			Map<NamedElement, Net> products) {
		this.trace = trace;
		this.products = products;
	}

	
	public void handleActivity(Activity act, Configuration conf) {
		Net net = new Net(act.getName());

		System.out.println("_______________"+net.getName());
		int i = 0;
		for (Element e : act.allOwnedElements()) {
//			System.out.println(e);
			String n = null;
			String mark = null;
			if (e instanceof InitialNode) {
				InitialNode in = (InitialNode) e;
				n = "initial"+ i++;
				mark = "1";
			} else if (e instanceof DecisionNode) {
				DecisionNode dn = (DecisionNode) e;
				n = "decision"+ i++;
			} else if (e instanceof MergeNode) {
				MergeNode dn = (MergeNode) e;
				n = "merge"+ i++;
			} else if (e instanceof ActivityFinalNode) {
				n = ((ActivityFinalNode)e).getName();
				if (n== null) 
					n = "final_"+ i++;
			} else if (e instanceof CallAction) {
				CallAction action = (CallAction) e;
				n = action.getName();
			} else if (e instanceof OpaqueAction) {
				OpaqueAction action = (OpaqueAction) e;
				n = action.getName();
			} else if (e instanceof SendSignalAction) {
				SendSignalAction send = (SendSignalAction) e;
				n = send.getName();
			} else if (e instanceof ReceiveSignalEvent) {
				ReceiveSignalEvent recv = (ReceiveSignalEvent) e;
				n = recv.getName();
			} else if (e instanceof AcceptEventAction) {
				AcceptEventAction recv = (AcceptEventAction) e;
				n = recv.getName();
			} else if (e instanceof ForkNode) {
				ForkNode fork = (ForkNode) e;
				n = fork.getName();
				if (n == null || n.matches("$\\s*^")) 
					n = "fork_"+i++;
				n+= e.hashCode();
				net.createTransition(n, Visibility.PRIVATE);
				trace.addTrace(e,n);
				continue;
			} else if (e instanceof JoinNode) {
				JoinNode join = (JoinNode) e;
				n = join.getName();
				if (n == null || n.matches("$\\s*^")) 
					n = "join_"+i++;
				n+= e.hashCode();
				net.createTransition(n, Visibility.PRIVATE);
				trace.addTrace(e, n);
				continue;
			}
			if (n == null) { 
				// System.err.println("unparsed state : "+e);
				continue;
			} else {
				n+= e.eClass().getName();
				n+= e.hashCode();
				net.createPlace(n);
				if (mark != null)
					net.setPlaceState(n, mark);
				trace.addTrace(e, n);
			}	
		}

		int trnum = 0;
		for (Element e : act.allOwnedElements()) {
			if (e instanceof ControlFlow) {
				ControlFlow t = (ControlFlow) e;
				String n = "transition_"+trnum++ ;;
				Visibility v = Visibility.PRIVATE;
//				if (n == null )
//				n = "transition_"+trnum++ ;
				if (t.getName() != null)
					n += t.getName();
				// handle join/fork
				ActivityNode src = t.getSource();
				ActivityNode dest = t.getTarget();
				String srcn = trace.getTarget(t.getSource());
				String destn = trace.getTarget(t.getTarget());
				if (srcn == null) {
					System.err.println("unknown source :"+src +";for edge :"+t);
					continue ;
				} else if (destn == null) {
					System.err.println("unknown target:"+dest +"; for edge :"+t);
					continue ;
				}

				if (src instanceof ForkNode || src instanceof JoinNode ) {
					// src gave rise to a transition
					net.createArc(destn, srcn, ArcType.OUTPUT);
				} else if (dest instanceof ForkNode || dest instanceof JoinNode ) {
					// dest gave rise to a transition
					net.createArc(srcn, destn, ArcType.INPUT);
				} else {
					if (src instanceof SendSignalAction
							|| src instanceof ReceiveSignalEvent
							|| src instanceof AcceptEventAction )
						v = Visibility.PUBLIC;
					else
						v = Visibility.PRIVATE;
					if (src instanceof DecisionNode 
							&& t.getGuard() != null 
							&& ! t.getGuard().stringValue().equals("true")) {
						n += " [" + t.getGuard().stringValue() + "]";
					}
					// basic pattern : create a transition and two arcs
					// per control flow edge
					n+= e.hashCode();
					net.createTransition(n,v);
					trace.addTrace(e, n);
					net.createArc(srcn, n, ArcType.INPUT);
					net.createArc(destn,n,ArcType.OUTPUT);
				}
			}
		}


//		System.out.println("_______________");
		products.put(act,net);
//		exhibit(net,conf);
	}
}
